import * as path from 'path';

import * as Connect_sqlite3 from 'connect-sqlite3';
import * as Express from 'express'; // https://www.npmjs.com/package/express
import * as Session from 'express-session'; // https://www.npmjs.com/package/express-session
import * as CreateError from 'http-errors'; // https://www.npmjs.com/package/http-errors
import * as Logger from 'morgan'; // https://www.npmjs.com/package/morgan
import * as Passport from 'passport';
import * as pluralize from 'pluralize';

import {router as Authentification_router} from "./Authentification.js";
import {router as Index_router} from "./Index.js";

const store = Connect_sqlite3(Session);

export const app = Express();

// 'index.ejs' uses 'pluralize'
// https://expressjs.com/fr/api.html#app.locals
app.locals.pluralize = pluralize; // Pluralize and singularize any word...

// Using template engines with Express (https://expressjs.com/en/guide/using-template-engines.html)
// '.ejs' files are transformed into HTML files after variables have been replaced by values...

app.set('views', path.join(__dirname, '..', "Web"));
app.set('view engine', 'ejs');

app.use(Express.json());
app.use(Express.urlencoded({extended: false}));
app.use(Express.static(path.join(__dirname, '..', "Web")));
app.use(Logger('dev'));
app.use(Session({
    secret: 'keyboard cat',
    resave: false,
    saveUninitialized: false,
    store: new store({db: 'sessions.db', dir: './Database'})
}));
app.use(Passport.authenticate('session'));

app.use('/', Index_router);
app.use('/', Authentification_router);

// Catch '404' and forward to error handler:
app.use((request, response, next) => {
    next(CreateError(404));
});

// Error handler:
app.use((error, request, response, next) => {
    // Set 'locals', only providing error in development:
    response.locals.message = error.message;
    response.locals.error = request.app.get('env') === 'development' ? error : {};

    response.status(error.status || 500);
    response.render('error');
});
