import * as crypto from 'crypto'; // https://nodejs.org/api/crypto.html

import * as Express from 'express';
import * as Passport from 'passport';
import * as Google_strategy from 'passport-google-oidc';

import {database, Credential, CREDENTIALS, USERS} from "./Database.js";

export const router = Express.Router();

router.get('/login', (request, response, next) => response.render('login')); // 'locals' is used...
// Google project name : 'Passport' (watt7212@gmail.com/anrwatt7)
router.get('/login/Google', Passport.authenticate('google'));
router.get('/oauth2/redirect/google', Passport.authenticate('google', {
    successRedirect: '/',
    failureRedirect: '/login'
}));
router.post('/logout', (request, response, next) => { // 'index.ejs' -> form => post
    request.logout(error => {
        if (error) return next(error);
        response.redirect('/');
    });
});
router.get('/signup', (request, response, next) => response.render('Signup'));
router.post('/signup', (request, response, next) => {
    const salt = crypto.randomBytes(16);
    crypto.pbkdf2(request.body.password, salt, 310000, 32, 'sha256', function (error, hashedPassword) {
        if (error) return next(error);
        database.run('INSERT INTO  ' + USERS + ' (username, hashed_password, salt) VALUES (?, ?, ?)', [
            request.body.username,
            hashedPassword,
            salt
        ], error => {
            if (error) return next(error);
            const user = {
                id: this.lastID,
                username: request.body.username
            };
            request.login(user, error => {
                if (error) return next(error);
                response.redirect('/');
            });
        });
    });
});

Passport.use(new Google_strategy({
    clientID: process.env['GOOGLE_CLIENT_ID'],
    clientSecret: process.env['GOOGLE_CLIENT_SECRET'],
    callbackURL: '/oauth2/redirect/google',
    scope: ['profile']
}, (issuer, profile, cb) => {
    database.get('SELECT * FROM ' + CREDENTIALS + ' WHERE provider = ? AND subject = ?', [
        issuer,
        profile.id
    ], (error, row: Credential) => {
        if (error) return cb(error);
        if (!row) {
            database.run('INSERT INTO ' + USERS + ' (name) VALUES (?)', [
                profile.displayName
            ], function (error) { // Keep 'function' (and not lambda)...
                if (error) return cb(error);
                const id = this.lastID; // 'this' object contains 'lastID' and 'changes' which contain the value of the last inserted row ID and the number of rows affected by this query respectively...
                database.run('INSERT INTO ' + CREDENTIALS + ' (user_id, provider, subject) VALUES (?, ?, ?)', [
                    id,
                    issuer,
                    profile.id
                ], error => {
                    if (error) return cb(error);
                    return cb(null, {id: id, name: profile.displayName});
                });
            });
        } else
            database.get('SELECT * FROM ' + USERS + ' WHERE id = ?', [
                row.user_id
            ], (error, row) => {
                if (error) return cb(error);
                if (!row) return cb(null, false);
                return cb(null, row);
            });
    });
}));
Passport.serializeUser((user, done) => { // https://nodejs.dev/en/learn/understanding-processnexttick/
    // <=> 'request.session.passport.user = {id: '..'}'
    // Calling 'setTimeout(() => {}, 0)' will execute it at the end of next tick, later than when using 'nextTick()'
    process.nextTick(() => done(null, {id: user.id, username: user.username, name: user.name}));
});
Passport.deserializeUser((user, done) => { // https://nodejs.dev/en/learn/understanding-processnexttick/
    // Attached to 'request' as 'request.user'
    // Calling 'setTimeout(() => {}, 0)' will execute it at the end of next tick, later than when using 'nextTick()'
    process.nextTick(() => done(null, user));
});
