import * as crypto from 'crypto'; // https://nodejs.org/api/crypto.html

import * as mkdirp from 'mkdirp';
import * as sqlite3 from 'sqlite3';

sqlite3.verbose();

mkdirp.sync("Database"); // Blocking...
export const database = new sqlite3.Database("Database/Database.sqlite3");
export const CREDENTIALS = "CREDENTIALS";
export const TASKS = "TASKS";
export const USERS = "USERS";

export interface Credential {
    id: number,
    user_id: number,
    provider: string,
    subject: string
}

export interface Task {
    id: number,
    owner_id: number,
    title: string,
    completed: number
}

database.serialize(() => {
    database.run("CREATE TABLE IF NOT EXISTS " + USERS + " ( \
    id INTEGER PRIMARY KEY, \
    username TEXT UNIQUE, \
    hashed_password BLOB, \
    salt BLOB, \
    name TEXT, \
    email TEXT UNIQUE, \
    email_verified INTEGER \
  )");

    database.run("CREATE TABLE IF NOT EXISTS " + CREDENTIALS + " ( \
    id INTEGER PRIMARY KEY, \
    user_id INTEGER NOT NULL, \
    provider TEXT NOT NULL, \
    subject TEXT NOT NULL, \
    UNIQUE (provider, subject) \
  )");

    database.run("CREATE TABLE IF NOT EXISTS " + TASKS + " ( \
    id INTEGER PRIMARY KEY, \
    owner_id INTEGER NOT NULL, \
    title TEXT NOT NULL, \
    completed INTEGER \
  )");

    // Initial user ('FranckBarbier', 'Bab64')
    const salt = crypto.randomBytes(16);
    database.run('INSERT OR IGNORE INTO ' + USERS + ' (username, hashed_password, salt) VALUES (?, ?, ?)', [
        'FranckBarbier',
        crypto.pbkdf2Sync('Bab64', salt, 310000, 32, 'sha256'),
        salt
    ]);
});
